/*
	*
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project ( https://gitlab.com/desq/ )
	* Qt5 example of analog clock, and Lenovo K8 Note's clock widget
	* are the inspiration for this. Some parts of the code are taken
	* from Qt5's example
	*
	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*
	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*
	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QtWidgets>

#include "Global.hpp"
#include "Widgets.hpp"
#include "Panel.hpp"
#include "StyleSheet.hpp"

#include <desqui/DesQShellPlugin.hpp>
#include <desqwl/DesQWlGlobal.hpp>

DesQPanel::DesQPanel( QWidget *parent ) : QWidget( parent ) {

	/* No frame, and stay at bottom */
	Qt::WindowFlags wFlags = Qt::FramelessWindowHint | Qt::WindowStaysOnBottomHint | Qt::BypassWindowManagerHint;

	setAttribute( Qt::WA_TranslucentBackground );

	/* Set the windowFlags */
	setWindowFlags( wFlags );

	setWindowTitle( tr( "DesQ Panel" ) );

	buildPanel();
	autoResize();

	loadSNI();

	connect( qApp->primaryScreen(), &QScreen::geometryChanged, this, &DesQPanel::autoResize );
};

void DesQPanel::autoResize() {

	// Span the entire width of the screen
	QSize scrSize = qApp->primaryScreen()->size();
	panelSize = QSize( scrSize.width(), shellSett->value( "PanelHeight" ) );

	setFixedSize( panelSize );

	// Move to the top left of the screen
	move( 0, 0 );
};

void DesQPanel::buildPanel() {

	int panelHeight = shellSett->value( "PanelHeight" );

	/* Application Switcher Button */
	/* ToDo: Implement the actual code here */
	appBtn = new AppButton( this );
	appBtn->setFixedHeight( panelHeight );

	/* Clock */
	clock = new ClockLabel( this );
	clock->setFixedHeight( panelHeight );

	tray = new TrayButton( this );
	tray->setFixedSize( panelHeight, panelHeight );

	info = new InfoButton( this );
	info->setFixedSize( panelHeight, panelHeight );
	connect(
		info, &QToolButton::clicked, [=]() {
			QMessageBox::information(
				nullptr,
				"DesQ | Feature unavailable",
				"This feature is currently unavailable. It will be coming soon."
			);
		}
	);

	power = new PowerButton( this );
	power->setFixedSize( panelHeight, panelHeight );
	connect(
		power, &QToolButton::clicked, [=]() {
			QMessageBox::information(
				nullptr,
				"DesQ | Feature unavailable",
				"This feature is currently unavailable. It will be coming soon."
			);
		}
	);

	QHBoxLayout *btnLyt = new QHBoxLayout();
	btnLyt->setSpacing( 0 );
	btnLyt->setContentsMargins( QMargins() );
	btnLyt->addWidget( tray );
	btnLyt->addWidget( info );
	btnLyt->addWidget( new QLabel( " " ) );
	btnLyt->addWidget( power );

	QGridLayout *page0lyt = new QGridLayout();
	page0lyt->setSpacing( 0 );
	page0lyt->setContentsMargins( QMargins() );

	page0lyt->addWidget( appBtn, 0, 0, Qt::AlignLeft );
	page0lyt->addWidget( clock, 0, 0, Qt::AlignCenter );
	page0lyt->addLayout( btnLyt, 0, 0, Qt::AlignRight );

	QWidget *page0 = new QWidget( this );
	page0->setLayout( page0lyt );

	baseLyt = new QStackedLayout();
	baseLyt->addWidget( page0 );
	baseLyt->setCurrentIndex( 0 );

	setLayout( baseLyt );
};

void DesQPanel::loadSNI() {

	/* SNI */
	QPluginLoader loader( "/usr/lib/desq/plugins/shell/libdesq-plugin-sni.so" );
	QObject *pObject = loader.instance();

	if ( pObject ) {
		DesQShellPlugin *plugin = qobject_cast<DesQShellPlugin *>(pObject);

		if ( !plugin )
			qWarning() << "Plugin Error:" << loader.errorString();

		else {
			tray->setEnabled( true );
			int pHeight = shellSett->value( "PanelHeight" );

			QWidget *sni = plugin->widget( nullptr );
			sni->setFixedHeight( pHeight );
			sni->setStyleSheet( ToolButtonStyle );
			connect(
				tray, &QToolButton::clicked, [=]() {
					baseLyt->setCurrentIndex( 1 );
				}
			);

			QToolButton *backBtn = new QToolButton();
			backBtn->setFixedSize( pHeight, pHeight );
			backBtn->setIcon( QIcon::fromTheme( "arrow-left" ) );
			backBtn->setStyleSheet( ToolButtonStyle );
			connect(
				backBtn, &QToolButton::clicked, [=]() {
					baseLyt->setCurrentIndex( 0 );
				}
			);

			QHBoxLayout *sniLyt = new QHBoxLayout();
			sniLyt->setSpacing( 0 );
			sniLyt->setContentsMargins( QMargins() );

			sniLyt->addStretch();
			sniLyt->addWidget( sni );
			sniLyt->addWidget( Spacer::horizontal( pHeight ) );
			sniLyt->addWidget( backBtn );

			QWidget *sniBase = new QWidget();
			sniBase->setLayout( sniLyt );

			baseLyt->addWidget( sniBase );

			qDebug() << sni->size();
		}
	}

	else {
		qWarning() << "Plugin Error:" << loader.errorString();
		qWarning() << "SNI will not be available";
	}
};

void DesQPanel::paintEvent( QPaintEvent *pEvent ) {

	QPainter painter( this );
	painter.setRenderHint( QPainter::Antialiasing );

	painter.save();
	painter.setBrush( QColor( 0, 0, 0, 230 ) );
	painter.setPen( Qt::NoPen );
	painter.drawRect( 0, 0, width(), height() );
	painter.restore();

	painter.end();

	QWidget::paintEvent( pEvent );
};

void DesQPanel::mousePressEvent( QMouseEvent *mEvent ) {

	mEvent->accept();
};

void DesQPanel::mouseMoveEvent( QMouseEvent *mEvent ) {

	mEvent->accept();
};

void DesQPanel::mouseReleaseEvent( QMouseEvent *mEvent ) {

	mEvent->accept();
};
